@@ -10,7 +10,7 @@
 mod types;
 
 use log::{debug, warn};
-use mac::{matches, _tt_as_expr_hack, unwrap_or_return};
+use mac::unwrap_or_return;
 use markup5ever::{local_name, namespace_prefix, namespace_url, ns};
 use std::borrow::Cow;
 use std::borrow::Cow::Borrowed;
@@ -18,7 +18,6 @@ use std::collections::btree_map::Iter;
 use std::collections::{BTreeMap, HashSet, VecDeque};
 use std::fmt::{Debug, Error, Formatter};
 use std::mem;
-use std::result::Result;
 
 pub use self::interface::{NextParserState, NodeOrText, Tracer, TreeSink};
 use self::types::*;
@@ -31,8 +30,8 @@ use crate::{LocalName, Namespace, Prefix};
 
 use crate::tendril::{StrTendril, Tendril};
 
-static XML_URI: &'static str = "http://www.w3.org/XML/1998/namespace";
-static XMLNS_URI: &'static str = "http://www.w3.org/2000/xmlns/";
+static XML_URI: &str = "http://www.w3.org/XML/1998/namespace";
+static XMLNS_URI: &str = "http://www.w3.org/2000/xmlns/";
 
 type InsResult = Result<(), Cow<'static, str>>;
 
@@ -41,11 +40,7 @@ struct NamespaceMapStack(Vec<NamespaceMap>);
 
 impl NamespaceMapStack {
     fn new() -> NamespaceMapStack {
-        NamespaceMapStack({
-            let mut vec = Vec::new();
-            vec.push(NamespaceMap::default());
-            vec
-        })
+        NamespaceMapStack(vec![NamespaceMap::default()])
     }
 
     fn push(&mut self, map: NamespaceMap) {
@@ -74,7 +69,7 @@ impl Debug for NamespaceMap {
     fn fmt(&self, f: &mut Formatter) -> Result<(), Error> {
         write!(f, "\nNamespaceMap[")?;
         for (key, value) in &self.scope {
-            write!(f, "   {:?} : {:?}\n", key, value)?;
+            writeln!(f, "   {:?} : {:?}", key, value)?;
         }
         write!(f, "]")
     }
@@ -113,11 +108,7 @@ impl NamespaceMap {
 
     #[doc(hidden)]
     pub fn insert(&mut self, name: &QualName) {
-        let prefix = if let Some(ref p) = name.prefix {
-            Some(p.clone())
-        } else {
-            None
-        };
+        let prefix = name.prefix.as_ref().cloned();
         let namespace = Some(Namespace::from(&*name.ns));
         self.scope.insert(prefix, namespace);
     }
@@ -176,15 +167,9 @@ impl NamespaceMap {
 }
 
 /// Tree builder options, with an impl for Default.
-#[derive(Copy, Clone)]
+#[derive(Copy, Clone, Default)]
 pub struct XmlTreeBuilderOpts {}
 
-impl Default for XmlTreeBuilderOpts {
-    fn default() -> XmlTreeBuilderOpts {
-        XmlTreeBuilderOpts {}
-    }
-}
-
 /// The XML tree builder.
 pub struct XmlTreeBuilder<Handle, Sink> {
     /// Configuration options for XmlTreeBuilder
@@ -211,9 +196,6 @@ pub struct XmlTreeBuilder<Handle, Sink> {
     /// Current namespace identifier
     current_namespace: NamespaceMap,
 
-    /// List of already present namespace local name attribute pairs.
-    present_attrs: HashSet<(Namespace, LocalName)>,
-
     /// Current tree builder phase.
     phase: XmlPhase,
 }
@@ -229,15 +211,14 @@ where
         let doc_handle = sink.get_document();
         XmlTreeBuilder {
             _opts: opts,
-            sink: sink,
-            doc_handle: doc_handle,
+            sink,
+            doc_handle,
             next_tokenizer_state: None,
             open_elems: vec![],
             curr_elem: None,
             namespace_stack: NamespaceMapStack::new(),
             current_namespace: NamespaceMap::empty(),
-            present_attrs: HashSet::new(),
-            phase: StartPhase,
+            phase: Start,
         }
     }
 
@@ -246,9 +227,11 @@ where
     pub fn trace_handles(&self, tracer: &dyn Tracer<Handle = Handle>) {
         tracer.trace_handle(&self.doc_handle);
         for e in self.open_elems.iter() {
-            tracer.trace_handle(&e);
+            tracer.trace_handle(e);
+        }
+        if let Some(h) = self.curr_elem.as_ref() {
+            tracer.trace_handle(h);
         }
-        self.curr_elem.as_ref().map(|h| tracer.trace_handle(&h));
     }
 
     // Debug helper
@@ -276,7 +259,7 @@ where
     }
 
     fn declare_ns(&mut self, attr: &mut Attribute) {
-        if let Err(msg) = self.current_namespace.insert_ns(&attr) {
+        if let Err(msg) = self.current_namespace.insert_ns(attr) {
             self.sink.parse_error(msg);
         } else {
             attr.name.ns = ns!(xmlns);
@@ -320,47 +303,58 @@ where
     // existing namespace context.
     //
     // Returns false if the attribute is a duplicate, returns true otherwise.
-    fn bind_attr_qname(&mut self, name: &mut QualName) -> bool {
+    fn bind_attr_qname(
+        &mut self,
+        present_attrs: &mut HashSet<(Namespace, LocalName)>,
+        name: &mut QualName,
+    ) -> bool {
         // Attributes don't have default namespace
         let mut not_duplicate = true;
 
         if name.prefix.is_some() {
             self.bind_qname(name);
-            not_duplicate = self.check_duplicate_attr(name);
+            not_duplicate = Self::check_duplicate_attr(present_attrs, name);
         }
         not_duplicate
     }
 
-    fn check_duplicate_attr(&mut self, name: &QualName) -> bool {
+    fn check_duplicate_attr(
+        present_attrs: &mut HashSet<(Namespace, LocalName)>,
+        name: &QualName,
+    ) -> bool {
         let pair = (name.ns.clone(), name.local.clone());
 
-        if self.present_attrs.contains(&pair) {
+        if present_attrs.contains(&pair) {
             return false;
         }
-        self.present_attrs.insert(pair);
+        present_attrs.insert(pair);
         true
     }
 
     fn process_namespaces(&mut self, tag: &mut Tag) {
+        // List of already present namespace local name attribute pairs.
+        let mut present_attrs: HashSet<(Namespace, LocalName)> = Default::default();
+
         let mut new_attr = vec![];
         // First we extract all namespace declarations
-        for mut attr in tag.attrs.iter_mut().filter(|attr| {
-            attr.name.prefix == Some(namespace_prefix!("xmlns")) ||
-                attr.name.local == local_name!("xmlns")
+        for attr in tag.attrs.iter_mut().filter(|attr| {
+            attr.name.prefix == Some(namespace_prefix!("xmlns"))
+                || attr.name.local == local_name!("xmlns")
         }) {
-            self.declare_ns(&mut attr);
+            self.declare_ns(attr);
         }
 
         // Then we bind those namespace declarations to attributes
         for attr in tag.attrs.iter_mut().filter(|attr| {
-            attr.name.prefix != Some(namespace_prefix!("xmlns")) &&
-                attr.name.local != local_name!("xmlns")
+            attr.name.prefix != Some(namespace_prefix!("xmlns"))
+                && attr.name.local != local_name!("xmlns")
         }) {
-            if self.bind_attr_qname(&mut attr.name) {
+            if self.bind_attr_qname(&mut present_attrs, &mut attr.name) {
                 new_attr.push(attr.clone());
             }
         }
-        mem::replace(&mut tag.attrs, new_attr);
+        tag.attrs = new_attr;
+
         // Then we bind the tags namespace.
         self.bind_qname(&mut tag.name);
 
@@ -383,6 +377,8 @@ where
 
         loop {
             let phase = self.phase;
+
+            #[allow(clippy::unused_unit)]
             match self.step(phase, token) {
                 Done => {
                     token = unwrap_or_return!(more_tokens.pop_front(), ());
@@ -409,13 +405,13 @@ where
                 return;
             },
 
-            tokenizer::DoctypeToken(d) => DoctypeToken(d),
-            tokenizer::PIToken(x) => PIToken(x),
-            tokenizer::TagToken(x) => TagToken(x),
-            tokenizer::CommentToken(x) => CommentToken(x),
-            tokenizer::NullCharacterToken => NullCharacterToken,
-            tokenizer::EOFToken => EOFToken,
-            tokenizer::CharacterTokens(x) => CharacterTokens(x),
+            tokenizer::DoctypeToken(d) => Doctype(d),
+            tokenizer::PIToken(x) => Pi(x),
+            tokenizer::TagToken(x) => Tag(x),
+            tokenizer::CommentToken(x) => Comment(x),
+            tokenizer::NullCharacterToken => NullCharacter,
+            tokenizer::EOFToken => Eof,
+            tokenizer::CharacterTokens(x) => Characters(x),
         };
 
         self.process_to_completion(token);
@@ -497,7 +493,7 @@ where
                 Some(expr) => expr,
                 None => Tendril::new(),
             }
-        };
+        }
         self.sink.append_doctype_to_document(
             get_tendril(doctype.name),
             get_tendril(doctype.public_id),
@@ -537,7 +533,7 @@ where
         P: Fn(ExpandedName) -> bool,
     {
         loop {
-            if self.current_node_in(|x| pred(x)) {
+            if self.current_node_in(&pred) {
                 break;
             }
             self.pop();
@@ -613,8 +609,8 @@ where
         self.debug_step(mode, &token);
 
         match mode {
-            StartPhase => match token {
-                TagToken(Tag {
+            Start => match token {
+                Tag(Tag {
                     kind: StartTag,
                     name,
                     attrs,
@@ -622,17 +618,17 @@ where
                     let tag = {
                         let mut tag = Tag {
                             kind: StartTag,
-                            name: name,
-                            attrs: attrs,
+                            name,
+                            attrs,
                         };
                         self.process_namespaces(&mut tag);
                         tag
                     };
-                    self.phase = MainPhase;
+                    self.phase = Main;
                     let handle = self.append_tag_to_doc(tag);
                     self.add_to_open_elems(handle)
                 },
-                TagToken(Tag {
+                Tag(Tag {
                     kind: EmptyTag,
                     name,
                     attrs,
@@ -640,26 +636,26 @@ where
                     let tag = {
                         let mut tag = Tag {
                             kind: EmptyTag,
-                            name: name,
-                            attrs: attrs,
+                            name,
+                            attrs,
                         };
                         self.process_namespaces(&mut tag);
                         tag
                     };
-                    self.phase = EndPhase;
+                    self.phase = End;
                     let handle = self.append_tag_to_doc(tag);
                     self.sink.pop(&handle);
                     Done
                 },
-                CommentToken(comment) => self.append_comment_to_doc(comment),
-                PIToken(pi) => self.append_pi_to_doc(pi),
-                CharacterTokens(ref chars) if !any_not_whitespace(chars) => Done,
-                EOFToken => {
+                Comment(comment) => self.append_comment_to_doc(comment),
+                Pi(pi) => self.append_pi_to_doc(pi),
+                Characters(ref chars) if !any_not_whitespace(chars) => Done,
+                Eof => {
                     self.sink
                         .parse_error(Borrowed("Unexpected EOF in start phase"));
-                    Reprocess(EndPhase, EOFToken)
+                    Reprocess(End, Eof)
                 },
-                DoctypeToken(d) => {
+                Doctype(d) => {
                     self.append_doctype_to_doc(d);
                     Done
                 },
@@ -669,9 +665,9 @@ where
                     Done
                 },
             },
-            MainPhase => match token {
-                CharacterTokens(chs) => self.append_text(chs),
-                TagToken(Tag {
+            Main => match token {
+                Characters(chs) => self.append_text(chs),
+                Tag(Tag {
                     kind: StartTag,
                     name,
                     attrs,
@@ -679,15 +675,15 @@ where
                     let tag = {
                         let mut tag = Tag {
                             kind: StartTag,
-                            name: name,
-                            attrs: attrs,
+                            name,
+                            attrs,
                         };
                         self.process_namespaces(&mut tag);
                         tag
                     };
                     self.insert_tag(tag)
                 },
-                TagToken(Tag {
+                Tag(Tag {
                     kind: EmptyTag,
                     name,
                     attrs,
@@ -695,8 +691,8 @@ where
                     let tag = {
                         let mut tag = Tag {
                             kind: EmptyTag,
-                            name: name,
-                            attrs: attrs,
+                            name,
+                            attrs,
                         };
                         self.process_namespaces(&mut tag);
                         tag
@@ -709,7 +705,7 @@ where
                         self.append_tag(tag)
                     }
                 },
-                TagToken(Tag {
+                Tag(Tag {
                     kind: EndTag,
                     name,
                     attrs,
@@ -717,8 +713,8 @@ where
                     let tag = {
                         let mut tag = Tag {
                             kind: EndTag,
-                            name: name,
-                            attrs: attrs,
+                            name,
+                            attrs,
                         };
                         self.process_namespaces(&mut tag);
                         tag
@@ -728,31 +724,31 @@ where
                     }
                     let retval = self.close_tag(tag);
                     if self.no_open_elems() {
-                        self.phase = EndPhase;
+                        self.phase = End;
                     }
                     retval
                 },
-                TagToken(Tag { kind: ShortTag, .. }) => {
+                Tag(Tag { kind: ShortTag, .. }) => {
                     self.pop();
                     if self.no_open_elems() {
-                        self.phase = EndPhase;
+                        self.phase = End;
                     }
                     Done
                 },
-                CommentToken(comment) => self.append_comment_to_tag(comment),
-                PIToken(pi) => self.append_pi_to_tag(pi),
-                EOFToken | NullCharacterToken => Reprocess(EndPhase, EOFToken),
-                DoctypeToken(_) => {
+                Comment(comment) => self.append_comment_to_tag(comment),
+                Pi(pi) => self.append_pi_to_tag(pi),
+                Eof | NullCharacter => Reprocess(End, Eof),
+                Doctype(_) => {
                     self.sink
                         .parse_error(Borrowed("Unexpected element in main phase"));
                     Done
                 },
             },
-            EndPhase => match token {
-                CommentToken(comment) => self.append_comment_to_doc(comment),
-                PIToken(pi) => self.append_pi_to_doc(pi),
-                CharacterTokens(ref chars) if !any_not_whitespace(chars) => Done,
-                EOFToken => self.stop_parsing(),
+            End => match token {
+                Comment(comment) => self.append_comment_to_doc(comment),
+                Pi(pi) => self.append_pi_to_doc(pi),
+                Characters(ref chars) if !any_not_whitespace(chars) => Done,
+                Eof => self.stop_parsing(),
                 _ => {
                     self.sink
                         .parse_error(Borrowed("Unexpected element in end phase"));
